@@ -9,12 +9,9 @@ from element_interface.utils import dict_to_uuid, find_full_path, find_root_dire
 
 from . import imaging_report, scan
 from .scan import (
+    get_image_files,
     get_imaging_root_data_dir,
-    get_nd2_files,
-    get_prairieview_files,
     get_processed_root_data_dir,
-    get_scan_box_files,
-    get_scan_image_files,
 )
 
 schema = dj.Schema()
@@ -270,9 +267,14 @@ class Preprocess(dj.Imported):
 
 @schema
 class ProcessingMethod(dj.Lookup):
-    definition = """
-    # Method, package, analysis suite used for processing of calcium imaging
-    # data (e.g. Suite2p, CaImAn, etc.)
+    """Package used for processing of calcium imaging data (e.g. Suite2p, CaImAn, etc.).
+
+    Attributes:
+        processing_method (str): Processing method.
+        processing_method_desc (str): Processing method description.
+    """
+
+    definition = """# Package used for processing of calcium imaging data (e.g. Suite2p, CaImAn, etc.).
     processing_method: char(8)
     ---
     processing_method_desc: varchar(1000)  # Processing method description
@@ -385,7 +387,7 @@ class MaskType(dj.Lookup):
     """Available labels for segmented masks (e.g. 'soma', 'axon', 'dendrite', 'neuropil').
 
     Attributes:
-        masky_type (str): Mask type.
+        mask_type (str): Mask type.
     """
 
     definition = """# Possible types of a segmented mask
@@ -405,13 +407,13 @@ class ProcessingTask(dj.Manual):
     This table defines a calcium imaging processing task for a combination of a
     `Scan` and a `ProcessingParamSet` entries, including all the inputs (scan, method,
     method's parameters). The task defined here is then run in the downstream table
-    Processing. This table supports definitions of both loading of pre-generated results
-    and the triggering of new analysis for all supported analysis methods
+    `Processing`. This table supports definitions of both loading of pre-generated results
+    and the triggering of new analysis for all supported analysis methods.
 
     Attributes:
-        Preprocess (foreign key):
-        ProcessingParamSet (foreign key):
-        processing_output_dir (str):
+        Preprocess (foreign key): Primary key from Preprocess.
+        ProcessingParamSet (foreign key): Primary key from ProcessingParamSet.
+        processing_output_dir (str): Output directory of the processed scan relative to the root data directory.
         task_mode (str): One of 'load' (load computed analysis results) or 'trigger'
             (trigger computation).
     """
@@ -441,16 +443,14 @@ class ProcessingTask(dj.Manual):
                 processed_dir / scan_dir / {processing_method}_{paramset_idx}
                 e.g.: sub4/sess1/scan0/suite2p_0
         """
-        image_locators = {
-            "NIS": get_nd2_files,
-            "ScanImage": get_scan_image_files,
-            "Scanbox": get_scan_box_files,
-            "PrairieView": get_prairieview_files,
-        }
-        image_locator = image_locators[(scan.Scan & key).fetch1("acq_software")]
+        acq_software = (scan.Scan & key).fetch1("acq_software")
+        filetypes = dict(
+            ScanImage="*.tif", Scanbox="*.sbx", NIS="*.nd2", PrairieView="*.tif"
+        )
 
         scan_dir = find_full_path(
-            get_imaging_root_data_dir(), image_locator(key)[0]
+            get_imaging_root_data_dir(),
+            get_image_files(key, filetypes[acq_software])[0],
         ).parent
         root_dir = find_root_directory(get_imaging_root_data_dir(), scan_dir)
 
@@ -588,7 +588,6 @@ class Processing(dj.Computed):
             else:
                 raise NotImplementedError("Unknown method: {}".format(method))
         elif task_mode == "trigger":
-
             method = (ProcessingParamSet * ProcessingTask & key).fetch1(
                 "processing_method"
             )
@@ -1433,27 +1432,27 @@ class Segmentation(dj.Computed):
                 MaskClassification.MaskType.insert(
                     cells, ignore_extra_fields=True, allow_direct_insert=True
                 )
-            elif method == "extract":
-                extract_dataset = imaging_dataset
-                masks = [
-                    dict(
-                        **key,
-                        segmentation_channel=0,
-                        mask=mask["mask_id"],
-                        mask_npix=mask["mask_npix"],
-                        mask_center_x=mask["mask_center_x"],
-                        mask_center_y=mask["mask_center_y"],
-                        mask_center_z=mask["mask_center_z"],
-                        mask_xpix=mask["mask_xpix"],
-                        mask_ypix=mask["mask_ypix"],
-                        mask_zpix=mask["mask_zpix"],
-                        mask_weights=mask["mask_weights"],
-                    )
-                    for mask in extract_dataset.load_results()
-                ]
+        elif method == "extract":
+            extract_dataset = imaging_dataset
+            masks = [
+                dict(
+                    **key,
+                    segmentation_channel=0,
+                    mask=mask["mask_id"],
+                    mask_npix=mask["mask_npix"],
+                    mask_center_x=mask["mask_center_x"],
+                    mask_center_y=mask["mask_center_y"],
+                    mask_center_z=mask["mask_center_z"],
+                    mask_xpix=mask["mask_xpix"],
+                    mask_ypix=mask["mask_ypix"],
+                    mask_zpix=mask["mask_zpix"],
+                    mask_weights=mask["mask_weights"],
+                )
+                for mask in extract_dataset.load_results()
+            ]
 
-                self.insert1(key)
-                self.Mask.insert(masks, ignore_extra_fields=True)
+            self.insert1(key)
+            self.Mask.insert(masks, ignore_extra_fields=True)
         else:
             raise NotImplementedError(f"Unknown/unimplemented method: {method}")
 
@@ -1694,8 +1693,7 @@ class Activity(dj.Computed):
         return suite2p_key_source.proj() + caiman_key_source.proj()
 
     def make(self, key):
-        """Populate the Activity with the results parsed from analysis
-        outputs."""
+        """Populate the Activity with the results parsed from analysis outputs."""
 
         method, imaging_dataset = get_loader_result(key, Curation)
 
@@ -1751,6 +1749,120 @@ class Activity(dj.Computed):
             raise NotImplementedError("Unknown/unimplemented method: {}".format(method))
 
 
+@schema
+class ProcessingQualityMetrics(dj.Computed):
+    """Quality metrics used to evaluate the results of the calcium imaging analysis pipeline.
+
+    Attributes:
+        Fluorescence (foreign key): Primary key from Fluorescence.
+    """
+
+    definition = """
+    -> Fluorescence
+    """
+
+    class Mask(dj.Part):
+        """Quality metrics used to evaluate the masks.
+
+        Attributes:
+            Fluorescence (foreign key): Primary key from Fluorescence.
+            Segmentation.Mask (foreign key): Primary key from Segmentation.Mask.
+            mask_area (float): Mask area in square micrometer.
+            roundness (float): Roundness between 0 and 1. Values closer to 1 are rounder.
+        """
+
+        definition = """
+        -> master
+        -> Segmentation.Mask
+        ---
+        mask_area=null: float  # Mask area in square micrometer.
+        roundness: float       # Roundness between 0 and 1. Values closer to 1 are rounder.
+        """
+
+    class Trace(dj.Part):
+        """Quality metrics used to evaluate the fluorescence traces.
+
+        Attributes:
+            Fluorescence (foreign key): Primary key from Fluorescence.
+            Fluorescence.Trace (foreign key): Primary key from Fluorescence.Trace.
+            skewness (float): Skewness of the fluorescence trace.
+            variance (float): Variance of the fluorescence trace.
+        """
+
+        definition = """
+        -> master
+        -> Fluorescence.Trace
+        ---
+        skewness: float   # Skewness of the fluorescence trace.
+        variance: float   # Variance of the fluorescence trace.
+        """
+
+    def make(self, key):
+        """Populate the ProcessingQualityMetrics table and its part tables."""
+        from scipy.stats import skew
+
+        (
+            mask_xpixs,
+            mask_ypixs,
+            mask_weights,
+            fluorescence,
+            fluo_channels,
+            mask_ids,
+            mask_npix,
+            px_height,
+            px_width,
+            um_height,
+            um_width,
+        ) = (Segmentation.Mask * scan.ScanInfo.Field * Fluorescence.Trace & key).fetch(
+            "mask_xpix",
+            "mask_ypix",
+            "mask_weights",
+            "fluorescence",
+            "fluo_channel",
+            "mask",
+            "mask_npix",
+            "px_height",
+            "px_width",
+            "um_height",
+            "um_width",
+        )
+
+        norm_mean = lambda x: x.mean() / x.max()
+        roundnesses = [
+            norm_mean(np.linalg.eigvals(np.cov(x, y, aweights=w)))
+            for x, y, w in zip(mask_xpixs, mask_ypixs, mask_weights)
+        ]
+
+        fluorescence = np.stack(fluorescence)
+
+        self.insert1(key)
+
+        self.Mask.insert(
+            dict(key, mask=mask_id, mask_area=mask_area, roundness=roundness)
+            for mask_id, mask_area, roundness in zip(
+                mask_ids,
+                mask_npix * (um_height / px_height) * (um_width / px_width),
+                roundnesses,
+            )
+        )
+
+        self.Trace.insert(
+            dict(
+                key,
+                fluo_channel=fluo_channel,
+                mask=mask_id,
+                skewness=skewness,
+                variance=variance,
+            )
+            for fluo_channel, mask_id, skewness, variance in zip(
+                fluo_channels,
+                mask_ids,
+                skew(fluorescence, axis=1),
+                fluorescence.std(axis=1),
+            )
+        )
+
+
 # ---------------- HELPER FUNCTIONS ----------------
 
 
